#include <avr/pgmspace.h>
#include <Arduino.h>
/*
 * Values in the tables are in KHz and repsents the band edge for antenna selection
 * Note:  The highest frequency is 64MHz
 * The most number of bands is 256,  since the index is that value
 */
const PROGMEM uint16_t FreqTable[] = { 
	0 , 1800, 2000, 3500,4000,5330,5410,7000,7300, 10100,10150,14000,14350,18068,18168,21000,21450,
	24890,24990,28000,29700,50000,54000, 0xffff
};

// for each band of frequencies (see FreqTable above ) what is the corresponding antenna.
const PROGMEM int8_t AntTable[] = {
	5 ,   1,   5,  2 ,  5,  5,  5,  3,  5,    1,   5,   4,   5,   4,   5,   4,   5,   4,   5,   4,   
	5,   4,    5,   5
};

//  what pins are used to select the antenna.  Least significant bit first
const PROGMEM int8_t AntPinMap[] = { 42, 43 , 44 };

byte
lookupAntennaIndex ( const uint16_t value, const byte guessIndex)
{
	uint8_t Ret = (guessIndex != 0xff) ? guessIndex : sizeof(FreqTable) / 4;
	uint8_t lowIndex = 0;
	uint8_t highIndex = sizeof(FreqTable) / 2;

	while (true)
	{
		uint16_t tableValueHigh = pgm_read_word( FreqTable + Ret + 1);
		uint16_t tableValueLow = pgm_read_word( FreqTable + Ret);
		if (tableValueLow > value )
			highIndex = Ret;
		else if ( tableValueHigh <= value )
			lowIndex = Ret + 1;
		else
			break;
		Ret = static_cast<uint8_t> ((static_cast<int16_t>(highIndex) + lowIndex) / 2);
		continue; 
	}
	return static_cast<byte>(Ret);
}

byte
freqToIndex(const unsigned long freq, const byte guessIndex)
{
	return lookupAntennaIndex( static_cast<uint16_t>(freq/1000), guessIndex);
}

byte
indexToAntenna(const byte index)
{
	return pgm_read_byte(AntTable+index);
}


// Moving the data to the pins with an external hardware decoder
void
outputAntennaExternalDecoding(const byte antenna)
{
	byte len = sizeof(AntPinMap);  // how many pins are there in the table
	for (byte i = 0 ; i < len ; ++i)
	{
		byte pin = pgm_read_byte(AntPinMap + i);
		pinMode(pin,OUTPUT);  // just in case
		digitalWrite(pin,bitRead(antenna,i) ? HIGH : LOW);
	}
}


// Moving the data to the pins using software to decode 
void
outputAntennaSoftwarelDecoding(const byte antenna)
{
	byte len = sizeof(AntPinMap);  // how many pins are there in the table
	for (byte i = 0 ; i < len ; ++i)
	{
		byte pin = pgm_read_byte(AntPinMap + i);
		pinMode(pin,OUTPUT);  // just in case
		digitalWrite(pin,antenna == i ? HIGH : LOW);
	}
}
