#ifndef ARDUINO_ICOMDEVICE_H_
#define ARDUINO_ICOMDEVICE_H_
#include <Arduino.h>

/**
* @author James Michener
*
*/

class IcomSerial;

class VFOState
{
public:
	VFOState(void) : 
		freq(0),
		mode(0xff),
		dataMode(0xff),
		bandwidth(0)
    {};
	uint32_t freq;
	uint8_t mode;  // 0xff means undefined
	uint8_t dataMode;  // 0x00 means Data mode off supports new command 26
	uint8_t bandwidth;
	void dump(const byte type);
};

const byte ICOMDEVTYPE_RADIOBIT = 0;
const byte ICOMDEVTYPE_BROADCASTBIT = 1;
const byte ICOMDEVTYPE_VFOSUB = 2;
const byte ICOMDEVTYPE_VFOSPLIT = 3;

/**
* This is a container class that stores all the information
* about each device found on the network
*/
class IcomDevice
{
public:
	IcomDevice(void) : 
    origID(0xff), 
	mappedID(0xff),
    typeFlag(0),  
    comm(NULL) 
    { }
	~IcomDevice(void){ }

	void dump(void);


	uint8_t origID;  // Device ID -  0xFF if unused
	uint8_t mappedID;  // if here is a duplicate ID
	/*
	* See above for a description of each bit in the type flag.
	*/
	uint8_t typeFlag;

	/// which serial port is it on
	IcomSerial * comm;
	/// if it ever told me a radio information then here it is
	VFOState main; // main VFO data
	VFOState sub;  // sub VFO data
};


#endif // ARDUINO_ICOMDEVICE_H_

