#include "IcomDeviceStatus.h"
#include "IcomMessage.h"
#include "IcomSerial.h"
extern const bool bThreePorts;

void IcomDeviceStatus::dump(void)
{
	if (!bThreePorts)
	{
		for (byte i = 0 ; i < maxIcomDevIndex ; ++i)
		{
			Serial3.print(i,DEC);
			Serial3.print(": ");
			icomDev[i].dump();
		}
	}
}



// @return the index into icomDev for device whose original ID is x
byte IcomDeviceStatus::findOrigID(const byte x)
{
	for (byte i = 0 ; i < maxIcomDevIndex; ++i)
	{
		if ( x == icomDev[i].origID )
			return i;
	}
	return 0xff;
}
// @return the index into icomDev for device whose mapped ID is x
byte IcomDeviceStatus::findMappedID(const byte x)
{
	for (byte i = 0; i < maxIcomDevIndex; ++i)
	{
		if (x == icomDev[i].mappedID)
			return i;
	}
	return 0xff;
}

bool IcomDeviceStatus::isIDOrigOrMapped(const uint8_t x)
{
	for (byte i = 0; i < maxIcomDevIndex; ++i)
	{
		if (x == icomDev[i].mappedID || x == icomDev[i].origID )
			return true;
	}
	return false;
}

byte
IcomDeviceStatus::findDev(IcomSerial * is)
{
	IcomMessage * imp = is->imReceive;
	imp->dump();  //dump the message
	byte sourceID = imp->getSourceID();
	byte i = 0;
	for ( ; i < maxIcomDevIndex ; ++i )
	{
		byte idx = icomDev[i].origID;
		if (idx == 0xff) break;
		// no duplicate device ID per port
		if (idx == sourceID  && icomDev[i].comm == is) 
			return i;
	}
	if (i == maxIcomDevIndex) // must be a new device  (new sourceID)
	{
		if (i == maxIcomDevices) 
			return 0xff;
		uint8_t mapID = sourceID;
		while (isIDOrigOrMapped(mapID) || mapID == 0xE7)  // reserve my ID
			++mapID;
		if (mapID != sourceID)
			icomDev[i].mappedID = mapID;
		icomDev[i].origID = sourceID;
		icomDev[i].comm = is;
		++maxIcomDevIndex;
	}
	//  icomDev[i].timeLastMessage = millis();
	return i;
}
