#ifndef ARDUINO_ICOMSERIAL
#define ARDUINO_ICOMSERIAL

#include <Arduino.h>

#include "IcomMessage.h"
#include "SimpleQueue.h"
#include "SimpleTimeout.h"

const unsigned long icomMessageReceiveTimeOutinMS = 100;  // was 120
const unsigned long icomMessageTransmitTimeOutinMS = 120; // was 170

/*
*  Constants to define the IcomSerialFlags
*/
/// on receive has a valid start character been received
const byte isValidStartFlag = 0;
/// is the serial port full duplex (true) or simplex (false)
const byte isDuplexFlag = 1;
/// am actually sending a message
const byte isTransmittingFlag = 2;
/// are we waiting after RX data before sending?
const byte isTXHoldOff = 3;
/// is this a new freq?
//const byte isNewFreq = 4;
/// has a radio on this port?
const byte hasRadioOnPort  = 5;


class IcomSerial
{
public:
	IcomSerial(void) : hs(NULL),
		imReceive(NULL),
		imTransmit(NULL),
		devID(' '),
		flags(0),
		m_lastin(0),
		m_transmitOffset(0)
	{ 
		stReceive.start(icomMessageReceiveTimeOutinMS );
		stTransmit.start(icomMessageTransmitTimeOutinMS ); 
	}
	~IcomSerial(void)
	{
	}

	// Check for serial port for received messages
	void checkReceive(void);
    void checkReceiveThrowAway();

	void checkTransmit(void);

	/**
	* @return 
	*  0 -  nothing to report
	*  1 -  Message timed out
	*  2 -  Message too long
	*  3 -  Message Received
	*  4 -  End of Message byte without a valid start
	*/
	byte add(const byte x);


	// which serial port is this?
	HardwareSerial * hs;

	// what is the current message that we are taking data for
	IcomMessage * imReceive;
	//  current message we are transmitting
	// if duplex,  then the message stays around until it has been received
	// if NULL then there is nothing to send.
	IcomMessage *imTransmit;

	// a human readable character to jog memory for serialport
	char devID;

	inline void flagSet(const byte x) { bitSet(flags,x); }
	inline void flagClear(const byte x) { bitClear(flags,x); }
	inline bool flagRead(const byte x) const { return bitRead(flags,x); }

	void queueIcomMessage( IcomMessage * x);

private:
	SimpleQueue<IcomMessage *,8 > transmitMessageQueue;
	SimpleTimeout stReceive;
	SimpleTimeout stTransmit;
	// have we received a valid start during receive
	byte flags;
	// last character received.  Used to detect the start sequence.
	byte m_lastin;

	byte m_transmitOffset;

};

#endif // ARDUINO_ICOMSERIAL

